import re
from . file import *

def load_stopwords(file_path):
    """
    :param file_path: Stop word file path
    :return: Stop word list
    """
    stopwords = [line.strip() for line in open(file_path, 'r', encoding='utf-8').readlines()]
    return stopwords

def remove_stopwords(sentence, stopwords):
    """
    :param sentence: List of sentences
    :param stopwords： Stop word list
    :return: Stop word list
    """
    words_list =[x for x in sentence if len(x) >1 and x not in stopwords]
    return words_list

def convert_list2str(list_data, space_mark = ''):
    """
    :param list_data: One-dimensional list data
    :param space_mark: The spacing symbol between string
    :return: List of converted strings
    """
    s = ""
    list_len = len(list_data)
    for i in range(list_len):
        data = list_data[i]
        s += str(data)
        if i < list_len - 1:
            s += space_mark
    return s

def convert_str2list(str_data, space_mark = ' '):
    """
    :param str_data: The data for the string to be converted is separated by spacek_mark
    :param space_mark: The spacing symbol between string
    :return: A list of strings, each of which is a string
    """
    return str_data.split(space_mark)

def load_synonym(file_path, space_mark='→', file_encoding='utf-8'):
    """
    :param file_path: Synonym file path
    :param space_mark: The spacing symbol between synonyms
    :param file_encoding: The encoding format of the file
    :return: List of synonyms. exp: [['abc', 'def'], ['tom', 'jack'],]
    """
    synonym = [line.strip().split(space_mark) for line in open(file_path, 'r', encoding=file_encoding).readlines()]
    return synonym

def __pair_in_list(pair, list_head):
    for item in list_head:
        if (pair[0] == item[0] and pair[1] == item[1]) or (pair[0] == item[1] and pair[1] == item[0]):
            return True
    return False

def wash_synonym(input_path, output_path, space_mark='→'):
    """
    :param input_path:  Synonym library file path
    :param output_path:  After cleaning the synonym library file path 
    :param space_mark: The spacing symbol between synonyms
    :return: None
    """
    synonym_list = load_synonym(input_path)    
    synonym_pair = []
    for pair in synonym_list:
        if not __pair_in_list(pair, synonym_pair):
            synonym_pair.append(pair)
        
    synonym_str = ""
    for pair in synonym_pair:
        synonym_str += pair[0] + space_mark + pair[1] + '\n'
    save_file(output_path, synonym_str)

def remove_synonym(sentence, synonym, right=False):
    """
    :param sentence: Sentence string, each word separated by a space. exp: "abc def tom jack"
    :param synonym: Synonym list. exp: [['abc', 'def'], ['tom', 'jack'], ...]
    :param right: By default, the value on the left is replaced by the value on the right, and if True is passed in, 
                the value on the right is replaced by the value on the left.
    :return: The string after converting the synonyms
    """
    word_list = convert_str2list(sentence.strip())
    list_range = range(len(word_list))
    if right:
        # trans right to left
        for group in synonym:
            for i in list_range:         
                if group[1] == word_list[i]:
                    word_list[i] = group[0]
    else:
        # trans left to right
        for group in synonym:
            for i in list_range:       
                if group[0] == word_list[i]:                  
                    word_list[i] = group[1]
    return convert_list2str(word_list, ' ')

def remove_website(buf):
    """
    :param buf: text buf
    :return: the text after del websites
    """
    re_rule = re.compile(r'[http|https]*://[a-zA-Z0-9.?/&=:]*', re.S)
    _buf = re.sub(re_rule, '', buf)
    return _buf



import setuptools

# Package meta-data.
NAME = 'pyftools'
DESCRIPTION = 'python file tools is a simple file and text processing library with batch documents.'
URL = 'https://github.com/hzcx998/pyftools'
EMAIL = '2323168280@qq.com'
AUTHOR = 'Jason Hu'
VERSION = '0.0.3'

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=URL,
    packages=setuptools.find_packages(),
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)


# -*- coding: utf-8 -*-
"""
Created on Wed Oct 21 11:17:48 2020

@author: Jason Hu
"""
import os
import sys
import chardet
import shutil

def get_text_charset(filename):
    """
    :param filename: Text file path
    :return: A character-encoded string for text
    """
    fp = open(file=filename, mode='rb') # 以二进制模式读取文件
    data = fp.read()
    fp.close()
    result = chardet.detect(data)
    #{'encoding': 'utf-8', 'confidence': 0.99, 'language': ''}
    return result['encoding']

def walk_files(path, func, arg):
    """
    walk all files in path, and call 'func' with arg in each file
    ----
    :path string: dir path
    :func callback: function callback
    :arg mixed: arg for func
    :return: return a list what 'func' return
    """
    retval = []
    lsdir = os.listdir(path)
    dirs = [n for n in lsdir if os.path.isdir(os.path.join(path, n))]
    if dirs:
        for n in dirs:
            retval += walk_files(os.path.join(path, n), func, arg) # recursion
    
    files = [n for n in lsdir if os.path.isfile(os.path.join(path, n))]
    for fname in files: 
        fpath = os.path.join(path, fname)
        data = func(fpath, arg)
        if data != None:
            retval.append(data)
    return retval

def __load_txt(path):
    try:
       fp = open(path, 'r',encoding=('utf-8'))
       data = fp.read()
       fp.close()
    except Exception:
        try:
            fp = open(path, 'r',encoding=('gbk'))
            data = fp.read()
            fp.close()
        except Exception:
            print("pyftools: open file", path, "failed!")
            print("pyftools: stop serverce!")
            sys.exit()
    return data

def load_file(path, types = None):
    """
    load file in path if file format in types list
    ----
    :param path: file path
    :param code: file type list, if None, load all files, or not load the files in the list, such as ['txt', 'xlsx']
    :return: a list is [path, data]
    """
    ext = path.split(".")[-1]
    if types != None:
        if ext not in types: # filter this file
            return None 
        
    if ext == "txt":
        return [path, __load_txt(path)]
    else:
        print("pyftools: format", ext, "not support!")
        return None

def load_files(path, types = None):   
    """
    load all files in path if file format in types list
    ----
    :param path: file path
    :param code: file type list, if None, load all files, or not load the files in the list, such as ['txt', 'xlsx']
    :return: a list, each item is [path, data]
    """  
    return walk_files(path, load_file, types)

def __convert_file_encoding(path, arg):
    data = __load_txt(path)
    with open(path, 'w+',encoding=(arg)) as fp:
        fp.write(data)

def convert_file_encoding(path, code):       
    """
    :param path: dir path
    :param code: the format will convert to. (utf-8, gbk...)
    :return: None
    """     
    walk_files(path, __convert_file_encoding, code)

def save_file(file_path, data, _encoding='utf-8'):
    """
    save data to file path
    ----
    :param file_path: file path
    :param data: file data
    :param _encoding: The code of the file to be saved
    :return: None
    """  
    with open(file_path, "w", encoding=_encoding) as fp:
        fp.write(data)

def wash_file(file_path, wash_arg):
    """
    clean one file with callback, and save the buf return from callback func
    ----
    :param file_path: file path dir
    :param callback: call back func. 
        exp:    def wash_func(pathname, arg):
                    ...
                    return None
    :return: None
    """    
    callback = wash_arg[0]
    arg = wash_arg[1]
    if callback == None:
        print("Error: pyftoolsf: wash_file: callback is None!")
        return
    file_data = load_file(file_path)
    if file_data == None:
        return
    buf = callback(file_data[0], file_data[1], arg)
    if buf != None:
        save_file(file_path, buf)

def wash_files(file_path, callback, arg):
    """
    clean one file with callback func
    ----
    :param file_path: file path dir
    :param callback: call back func. 
        exp:    def wash_func(pathname, arg):
                    ...
                    return None
    :return: None
    """    
    walk_files(file_path, wash_file, [callback, arg])

def get_filename(pathname, split_char='/'):
    """
    :param pathname: path name
    :param split_char: Path interval character
    :return: file name
    """
    pathname = pathname.split(split_char)
    return pathname[-1]

def remove_files(filepath, file_list = None):
    """
    Deletes all files and folders in a directory
    ----
    :param filepath: The path to the file to delete if not ""
    :param file_list: Remove files in file if file_list not None
    :return: None
    """
    if filepath != None:
        del_list = os.listdir(filepath)
        for f in del_list:
            file_path = os.path.join(filepath, f)
            if os.path.isfile(file_path):
                os.remove(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
    if file_list != None:
        for file in file_list:
            os.remove(file)

def copy_file(src, dest):
    filename = get_filename(src, '\\') 
    shutil.copyfile(src, dest + '/' + filename)

def copy_files(src, dest):
    walk_files(src, copy_file, dest)